# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .price import Price
from .variant import Variant
from .._models import BaseModel
from .availability_status import AvailabilityStatus

__all__ = ["ProductRetrieveResponse"]


class ProductRetrieveResponse(BaseModel):
    id: str

    availability: AvailabilityStatus

    price: Price

    title: str

    url: str

    brand_id: Optional[str] = None

    brand_name: Optional[str] = None

    description: Optional[str] = None

    gender: Optional[Literal["male", "female", "unisex"]] = None

    image_urls: Optional[List[str]] = None

    key_features: Optional[List[str]] = None

    materials: Optional[List[str]] = None

    variants: Optional[List[Variant]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import TypeAlias

from .price import Price
from .variant import Variant
from .._models import BaseModel
from .availability_status import AvailabilityStatus

__all__ = ["SearchPerformResponse", "SearchPerformResponseItem"]


class SearchPerformResponseItem(BaseModel):
    id: str

    availability: AvailabilityStatus

    brand_name: str

    image_url: str

    price: Price

    score: int

    title: str

    url: str

    description: Optional[str] = None

    variants: Optional[List[Variant]] = None


SearchPerformResponse: TypeAlias = List[SearchPerformResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .brand import Brand as Brand
from .price import Price as Price
from .variant import Variant as Variant
from .brand_list_params import BrandListParams as BrandListParams
from .availability_status import AvailabilityStatus as AvailabilityStatus
from .brand_list_response import BrandListResponse as BrandListResponse
from .search_perform_params import SearchPerformParams as SearchPerformParams
from .search_perform_response import SearchPerformResponse as SearchPerformResponse
from .enrich_enrich_url_params import EnrichEnrichURLParams as EnrichEnrichURLParams
from .product_retrieve_response import ProductRetrieveResponse as ProductRetrieveResponse
from .enrich_enrich_url_response import EnrichEnrichURLResponse as EnrichEnrichURLResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .price import Price
from .._models import BaseModel

__all__ = ["EnrichEnrichURLResponse"]


class EnrichEnrichURLResponse(BaseModel):
    main_image_url: str

    other_image_urls: List[str]

    price: Price

    title: str

    url: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["Brand"]


class Brand(BaseModel):
    id: str

    name: str

    description: Optional[str] = None

    logo_url: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .brand import Brand
from .._models import BaseModel

__all__ = ["BrandListResponse", "Pagination"]


class Pagination(BaseModel):
    current_page: int

    page_size: int

    total_count: int

    total_pages: int


class BrandListResponse(BaseModel):
    items: List[Brand]

    pagination: Pagination
    """Pagination metadata for responses"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..types import search_perform_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.search_perform_response import SearchPerformResponse

__all__ = ["SearchResource", "AsyncSearchResource"]


class SearchResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SearchResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/channel3-ai/sdk-python#accessing-raw-response-data-eg-headers
        """
        return SearchResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SearchResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/channel3-ai/sdk-python#with_streaming_response
        """
        return SearchResourceWithStreamingResponse(self)

    def perform(
        self,
        *,
        base64_image: Optional[str] | NotGiven = NOT_GIVEN,
        config: search_perform_params.Config | NotGiven = NOT_GIVEN,
        context: Optional[str] | NotGiven = NOT_GIVEN,
        filters: search_perform_params.Filters | NotGiven = NOT_GIVEN,
        image_url: Optional[str] | NotGiven = NOT_GIVEN,
        limit: Optional[int] | NotGiven = NOT_GIVEN,
        query: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SearchPerformResponse:
        """
        Search for products.

        Args:
          base64_image: Base64 encoded image

          config: Optional configuration

          context: Optional customer information to personalize search results

          filters: Optional filters

          image_url: Image URL

          limit: Optional limit on the number of results

          query: Search query

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v0/search",
            body=maybe_transform(
                {
                    "base64_image": base64_image,
                    "config": config,
                    "context": context,
                    "filters": filters,
                    "image_url": image_url,
                    "limit": limit,
                    "query": query,
                },
                search_perform_params.SearchPerformParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SearchPerformResponse,
        )


class AsyncSearchResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSearchResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/channel3-ai/sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSearchResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSearchResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/channel3-ai/sdk-python#with_streaming_response
        """
        return AsyncSearchResourceWithStreamingResponse(self)

    async def perform(
        self,
        *,
        base64_image: Optional[str] | NotGiven = NOT_GIVEN,
        config: search_perform_params.Config | NotGiven = NOT_GIVEN,
        context: Optional[str] | NotGiven = NOT_GIVEN,
        filters: search_perform_params.Filters | NotGiven = NOT_GIVEN,
        image_url: Optional[str] | NotGiven = NOT_GIVEN,
        limit: Optional[int] | NotGiven = NOT_GIVEN,
        query: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SearchPerformResponse:
        """
        Search for products.

        Args:
          base64_image: Base64 encoded image

          config: Optional configuration

          context: Optional customer information to personalize search results

          filters: Optional filters

          image_url: Image URL

          limit: Optional limit on the number of results

          query: Search query

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v0/search",
            body=await async_maybe_transform(
                {
                    "base64_image": base64_image,
                    "config": config,
                    "context": context,
                    "filters": filters,
                    "image_url": image_url,
                    "limit": limit,
                    "query": query,
                },
                search_perform_params.SearchPerformParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SearchPerformResponse,
        )


class SearchResourceWithRawResponse:
    def __init__(self, search: SearchResource) -> None:
        self._search = search

        self.perform = to_raw_response_wrapper(
            search.perform,
        )


class AsyncSearchResourceWithRawResponse:
    def __init__(self, search: AsyncSearchResource) -> None:
        self._search = search

        self.perform = async_to_raw_response_wrapper(
            search.perform,
        )


class SearchResourceWithStreamingResponse:
    def __init__(self, search: SearchResource) -> None:
        self._search = search

        self.perform = to_streamed_response_wrapper(
            search.perform,
        )


class AsyncSearchResourceWithStreamingResponse:
    def __init__(self, search: AsyncSearchResource) -> None:
        self._search = search

        self.perform = async_to_streamed_response_wrapper(
            search.perform,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from channel3_sdk import Channel3, AsyncChannel3
from channel3_sdk.types import SearchPerformResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSearch:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_perform(self, client: Channel3) -> None:
        search = client.search.perform()
        assert_matches_type(SearchPerformResponse, search, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_perform_with_all_params(self, client: Channel3) -> None:
        search = client.search.perform(
            base64_image="base64_image",
            config={
                "enrich_query": True,
                "semantic_search": True,
            },
            context="context",
            filters={
                "availability": ["InStock"],
                "brand_ids": ["string"],
                "gender": "male",
                "price": {
                    "max_price": 0,
                    "min_price": 0,
                },
            },
            image_url="image_url",
            limit=0,
            query="query",
        )
        assert_matches_type(SearchPerformResponse, search, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_perform(self, client: Channel3) -> None:
        response = client.search.with_raw_response.perform()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        search = response.parse()
        assert_matches_type(SearchPerformResponse, search, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_perform(self, client: Channel3) -> None:
        with client.search.with_streaming_response.perform() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            search = response.parse()
            assert_matches_type(SearchPerformResponse, search, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncSearch:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_perform(self, async_client: AsyncChannel3) -> None:
        search = await async_client.search.perform()
        assert_matches_type(SearchPerformResponse, search, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_perform_with_all_params(self, async_client: AsyncChannel3) -> None:
        search = await async_client.search.perform(
            base64_image="base64_image",
            config={
                "enrich_query": True,
                "semantic_search": True,
            },
            context="context",
            filters={
                "availability": ["InStock"],
                "brand_ids": ["string"],
                "gender": "male",
                "price": {
                    "max_price": 0,
                    "min_price": 0,
                },
            },
            image_url="image_url",
            limit=0,
            query="query",
        )
        assert_matches_type(SearchPerformResponse, search, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_perform(self, async_client: AsyncChannel3) -> None:
        response = await async_client.search.with_raw_response.perform()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        search = await response.parse()
        assert_matches_type(SearchPerformResponse, search, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_perform(self, async_client: AsyncChannel3) -> None:
        async with async_client.search.with_streaming_response.perform() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            search = await response.parse()
            assert_matches_type(SearchPerformResponse, search, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional
from typing_extensions import Literal, TypedDict

from .._types import SequenceNotStr
from .availability_status import AvailabilityStatus

__all__ = ["SearchPerformParams", "Config", "Filters", "FiltersPrice"]


class SearchPerformParams(TypedDict, total=False):
    base64_image: Optional[str]
    """Base64 encoded image"""

    config: Config
    """Optional configuration"""

    context: Optional[str]
    """Optional customer information to personalize search results"""

    filters: Filters
    """Optional filters"""

    image_url: Optional[str]
    """Image URL"""

    limit: Optional[int]
    """Optional limit on the number of results"""

    query: Optional[str]
    """Search query"""


class Config(TypedDict, total=False):
    enrich_query: bool

    semantic_search: bool


class FiltersPrice(TypedDict, total=False):
    max_price: Optional[float]
    """Maximum price, in dollars and cents"""

    min_price: Optional[float]
    """Minimum price, in dollars and cents"""


class Filters(TypedDict, total=False):
    availability: Optional[List[AvailabilityStatus]]
    """List of availability statuses"""

    brand_ids: Optional[SequenceNotStr[str]]
    """List of brand IDs"""

    gender: Optional[Literal["male", "female", "unisex"]]

    price: Optional[FiltersPrice]
    """Price filter. Values are inclusive."""
